DECLARE SUB glEnd ()
DECLARE SUB vLine (x%, ytop%, ybot%, c%)
DECLARE SUB vbox (x1%, ytop%, x2%, ybot%, c%)
DECLARE SUB vPSET (x%, y%, c%)
DECLARE SUB glBegin ()
DEFINT A-Z
'--------------------------------------------------
' QB Voxel Engine 2.0 w/ rotations
' by Toshihiro Horie 1998, 2000
' updated with double buffer and vertical occlusion buffers.
' http://www.ocf.berkeley.edu/~horie/project.html
'--------------------------------------------------

DECLARE SUB Precalc ()
DECLARE SUB SetPal ()

CONST xmax = 320, ymax = 200: 'screen resolution
CONST xc = 160, yc = 200:     'center of rotation
CONST xh = 64, yh = 64:       'width and height of 2d heightfield
CONST xhc = 32, yhc = 32:     'half the width and height of heightfield
CONST dar = 35: ' this makes the voxels look taller
CONST mode = 2: ' Mode 1: dots-- fastest but not  voxel graphics
                ' Mode 2: uses thick -- voxels (with sbuffer)
                ' Mode 3: uses rectangles (real voxels w/o interpolation)
                ' Mode 4: uses lines (pseudo voxels)

DIM SHARED land(xh, yh)
DIM SHARED sine(64)
DIM SHARED cosine(64)
DIM SHARED c(128)
DIM SHARED s(128)
DIM SHARED xtable(10 TO 73, -32 TO 31)
DIM SHARED ybottable(1 TO 255)
DIM SHARED ytoptable(10 TO 73, 0 TO 255)
DIM SHARED ytab(200)
DIM SHARED filly(320) 'vertical occlusion buffer (s-buffer)
COMMON SHARED screensize%
screensize% = 32003
DIM SHARED vscreen%(screensize%)


Precalc
SCREEN 13: CLS
GET (0, 0)-(319, 199), vscreen%

SetPal

'draw 2-d color height map on right corner

LOCATE 2, 17: COLOR 123: PRINT "QB VOXEL ENGINE 2.0"
LOCATE 3, 19: COLOR 123: PRINT "Toshihiro Horie"


'---------
'Main loop
'---------
FOR a = 0 TO 120
Ca = c(a)
Sa = s(a)

CALL glBegin
'FOR j = 0 TO xh - 1
'FOR i = 0 TO xh - 1
'   CALL vPSET(i, j, land(i, j))
'NEXT: NEXT

FOR z3d = 40 TO 63 'draw front to back to get less overdraw using sbuffer
   z = z3d + 10: 'z ranges from 10 to 73
   yo = z3d - 42
   yrot2 = yo * Sa \ 512
   yrot1 = yo * Ca \ 512
  
   ybot = ybottable(z)
     
   FOR x3d = -32 TO 30
      xo = x3d
      'only needed if rotating mountains
      xr = xo * Ca \ 512 + yrot2 + xhc
      yr = yrot1 + yhc - xo * Sa \ 512
      'check if we are inside the boundaries of our heightmap
      IF xr < 0 OR xr > xh - 1 OR yr < 0 OR yr > yh - 1 THEN
         GOTO nodraw
         y3d = 0
      ELSE
         y3d = land(xr, yr)
      END IF
      colour = y3d
      ytop = ytoptable(z, y3d)
      x = xtable(z, x3d)
      SELECT CASE mode
         CASE 1
            CALL vPSET(x, ytop, colour)
         CASE 2
            CALL vLine(x, ytop, ybot, colour)
            CALL vLine(x + 1, ytop, ybot, colour - 2)
            CALL vLine(x - 1, ytop, ybot, colour)
         CASE 4
            CALL vLine(x, ytop, ybot, colour)
         CASE 3
            x2 = xtable(z, x3d + 1): CALL vbox(x, ytop, x2, ybot, colour)
     END SELECT
nodraw:
   NEXT
NEXT
k$ = INKEY$
IF k$ = CHR$(27) THEN CLEAR : END
CALL glEnd
NEXT

SUB glBegin
DEF SEG = VARSEG(vscreen%(0))
p0 = PEEK(0)
p1 = PEEK(1)
p2 = PEEK(2)
p3 = PEEK(3)
REDIM vscreen%(screensize%)
DEF SEG = VARSEG(vscreen%(0))
POKE 0, p0
POKE 1, p1
POKE 2, p2
POKE 3, p3

IF mode = 2 THEN
FOR i = 0 TO 319
filly(i) = 199
NEXT i
END IF
END SUB

SUB glEnd
PUT (0, 0), vscreen%, PSET
END SUB

SUB Precalc
deg2rad# = 3.1415926535# / 32

FOR i = 0 TO 319
filly(i) = 319
NEXT i
FOR i = 0 TO 199
y& = 320& * i
IF y& > 32767 THEN y& = y& - 65536
ytab(i) = INT(y&)
NEXT i

FOR i = 0 TO 63
   sine(i) = INT(SIN(i * deg2rad#) * 40) + 40
   cosine(i) = INT(COS(i * deg2rad#) * 40) + 40
NEXT i

'set up sinus tables
pi# = 3.1415926535#
deg2bin# = 2 * pi# / 128
scale = 512
FOR theta = 0 TO 63
    s(theta) = CINT(SIN(theta * deg2bin#) * scale)
    c(theta) = CINT(COS(theta * deg2bin#) * scale)
NEXT
FOR theta = 64 TO 127
    s(theta) = -s(theta - 63)
    c(theta) = c(127 - theta)
NEXT


FOR j = 0 TO yh - 1
FOR i = 0 TO xh - 1
   land(i, j) = sine(i) + cosine(j) + 10
NEXT: NEXT
'precalc
FOR z = 10 TO 73
FOR x3d = -32 TO 31
   xtable(z, x3d) = 160 * x3d / z + xc
NEXT: NEXT
'precalc ybot
FOR z = 1 TO 255
        yb = yc + dar / z
        IF yb > 199 THEN yb = 199
        ybottable(z) = yb
NEXT z
'precalc ytop
FOR z = 10 TO 73
FOR y3d = 0 TO 255
      ytoptable(z, y3d) = yc - dar * y3d / z
NEXT: NEXT

END SUB

SUB SetPal
FOR i = 0 TO 63
OUT &H3C8, i + 0
OUT &H3C9, 0
OUT &H3C9, i
OUT &H3C9, 0
OUT &H3C8, i + 64
OUT &H3C9, i
OUT &H3C9, i
OUT &H3C9, i
OUT &H3C8, i + 128
OUT &H3C9, i
OUT &H3C9, i \ 2
OUT &H3C9, 0
OUT &H3C8, i + 196
OUT &H3C9, i
OUT &H3C9, i
OUT &H3C9, 0
NEXT
OUT &H3C8, 0
OUT &H3C9, 0
OUT &H3C9, 0
OUT &H3C9, 0
END SUB

SUB vbox (x1, ytop, x2, ybot, c)
'IF (ybot > 199) THEN BEEP
' there seems to be a bug in the sbuffering code
' that prevents this mode from displaying correctly
' so it is commented out. (TH 2000)

'IF ytop < ybot THEN SWAP ytop, ybot
'IF filly(x1) < ybot THEN ybot = filly(x1)
'IF filly(x1) <= ytop THEN
'        'completely obscured line
'        EXIT SUB
'ELSE
'        ' top shows through
'        FOR x = x1 TO x2
'        filly(x) = ytop
'        NEXT
'END IF

xw = x2 - x1
p& = 4 + ytop * 320& + x1
FOR y = ytop TO ybot
    FOR x = 0 TO xw
       POKE p& + x, c
    NEXT x
    p& = p& + 320
NEXT y
END SUB

SUB vLine (x, ytop, ybot, c)
IF ybot < ytop THEN SWAP ytop, ybot

IF filly(x) < ybot THEN ybot = filly(x)

IF filly(x) <= ytop THEN
        'completely obscured line
        EXIT SUB
ELSE
        ' top shows through
       filly(x) = ybot
END IF

p& = 4 + 320& * ytop + x
FOR y = ytop TO ybot
    POKE p&, c: p& = p& + 320
NEXT y

END SUB

SUB vPSET (x, y, c)
'IF (y > 199) THEN
'        EXIT SUB
'ELSEIF (y < 0) THEN
'        EXIT SUB
'END IF
'IF (x > 319) THEN
'        EXIT SUB
'ELSEIF (x < 0) THEN
'        EXIT SUB
'END IF
POKE 4 + ytab(y) + x, c
'POKE 4 + y * 320& + x, c
END SUB

