DEFINT A-Z
'=============================================================
' QB Voxel Engine 2.1d w/ rotations
' by Toshihiro Horie revised 01/07/1998
' http://www.ocf.berkeley.edu/~horie/project.html
' ----------------------------------------------------------
' * Uses the Angelo's DirectQB Library.
' * Put this in your CONFIG.SYS:
'   DEVICE=C:\WINDOWS\HIMEM.SYS
'   DEVICE=C:\WINDOWS\EMM386.EXE 4096 RAM
' * You need DQB.LIB and DQB.QLB to run this.
' * If they are located in C:\QB45\LIB\, then
'   load QB v4.5 using QB /LC:\QB45\LIB\DQB.
'=============================================================

DECLARE SUB precalc ()
DECLARE SUB SetPal ()
'$INCLUDE: 'c:\qb45\dqb\DIRECTQB.BI'

CONST xmax = 320, ymax = 200: 'screen resolution
CONST xc = 160, yc = 194:     'center of rotation
CONST xh = 64, yh = 64:       'width and height of 2d heightfield
CONST xhc = 32, yhc = 32:     'half the width and height of heightfield
CONST dar = 35: ' this makes the voxels look taller
CONST Mode = 3: ' Mode 1: uses dots -- fastest but not voxel graphics
                ' Mode 2: uses lines-- looks okay
                ' Mode 3: uses boxes-- real voxels


DIM SHARED land(xh, yh)
DIM SHARED sine(64)
DIM SHARED cosine(64)
DIM SHARED c(128)
DIM SHARED s(128)
DIM SHARED xtable(10 TO 73, -32 TO 31)
DIM SHARED ybottable(1 TO 255)
DIM SHARED ytoptable(1 TO 73, 0 TO 255)


IF DQBinit(1, 0) THEN
   PRINT "DQB init error: Need more EMS memory."
   PRINT "Press any key to exit."
   k$ = INPUT$(1)
   DQBclose
   END
END IF

'***********
CONST layer1 = 1
'***********


precalc
DQBinitVGA
DQBclearLayer layer1
DQBprint layer1, "QB VOXEL ENGINE 2.1 DQB", 110, 20, 123
DQBprint layer1, "by Toshihiro Horie", 124, 30, 123
SetPal

'draw 2-d color height map on right corner
FOR j = 0 TO xh - 1
FOR i = 0 TO xh - 1
   DQBpset layer1, i, j, land(i, j)
NEXT: NEXT
'DQBLine layer1, 0, 199, 319, 199, 97

'---------
'Main loop
'---------
DO
FOR a = 0 TO 127
Ca = c(a)
Sa = s(a)
'clear window
DQBboxf layer1, 32, 84, xmax - 35, yc + 1, 0
FOR z3d = 63 TO 40 STEP -1
   z = z3d + 10: 'z ranges from 10 to 73
   yo = z3d - 42
   yrot2 = yo * Sa \ 512
   yrot1 = yo * Ca \ 512
   ybot = ybottable(z)
   FOR x3d = -32 TO 30
      xo = x3d
      'only needed if rotating mountains
      xr = xo * Ca \ 512 + yrot2 + xhc
      yr = yrot1 + yhc - xo * Sa \ 512
      'check if we are inside the boundaries of our heightmap
      IF xr < 0 OR xr > xh - 1 OR yr < 0 OR yr > yh - 1 THEN
         GOTO nodraw
         y3d = 0
      ELSE
         y3d = land(xr, yr)
      END IF
      colour = y3d
      ytop = ytoptable(z, y3d)
      x = xtable(z, x3d)
      SELECT CASE Mode
         CASE 1
            DQBpset layer1, x, ytop, colour
         CASE 2
            DQBLine layer1, x, ytop, x, ybot, colour
            IF z3d = 40 THEN  'make front voxels fatter
               DQBLine layer1, x + 1, ytop, x + 1, 199, colour - 2
               DQBLine layer1, x - 1, ytop, x - 1, 199, colour
            END IF
         CASE 3
            x2 = xtable(z, x3d + 1) - 1
            DQBboxf layer1, x, ytop, x2, ybot, colour
     END SELECT
nodraw:
   NEXT
IF INKEY$ > "" THEN GOTO cleanup
NEXT
DQBcopyLayer layer1, VIDEO: 'blit buffer to screen
NEXT
LOOP
cleanup:
DQBinitText
DQBclose
END

REM $DYNAMIC
SUB precalc
deg2rad# = 3.1415926535# / 32
FOR i = 0 TO 63
   sine(i) = INT(SIN(i * deg2rad#) * 40) + 40
   cosine(i) = INT(COS(i * deg2rad#) * 40) + 40
NEXT i

'set up sinus tables
pi# = 3.1415926535#
deg2bin# = 2 * pi# / 128
scale = 512
FOR theta = 0 TO 63
    s(theta) = CINT(SIN(theta * deg2bin#) * scale)
    c(theta) = CINT(COS(theta * deg2bin#) * scale)
NEXT
FOR theta = 64 TO 127
    s(theta) = -s(theta - 63)
    c(theta) = c(127 - theta)
NEXT


FOR j = 0 TO yh - 1
FOR i = 0 TO xh - 1
   land(i, j) = sine(i) + cosine(j) + 1
NEXT: NEXT
'precalc
FOR z = 10 TO 73
FOR x3d = -32 TO 31
   xtable(z, x3d) = 160 * x3d / z + xc
NEXT: NEXT
'precalc ybot
FOR z = 1 TO 255
ybottable(z) = yc + dar / z
NEXT z
'precalc ytop
FOR z = 1 TO 73
FOR y3d = 0 TO 255
      ytoptable(z, y3d) = yc - dar * y3d / z
NEXT: NEXT

END SUB

SUB SetPal
FOR i = 0 TO 63
OUT &H3C8, i + 0
OUT &H3C9, 0
OUT &H3C9, i
OUT &H3C9, 0
OUT &H3C8, i + 64
OUT &H3C9, i
OUT &H3C9, i
OUT &H3C9, i
OUT &H3C8, i + 128
OUT &H3C9, i
OUT &H3C9, i \ 2
OUT &H3C9, 0
OUT &H3C8, i + 196
OUT &H3C9, 0
OUT &H3C9, 0
OUT &H3C9, i * 2 \ 3
NEXT
OUT &H3C8, 0
OUT &H3C9, 0
OUT &H3C9, 0
OUT &H3C9, 0
END SUB

