DEFINT A-Z
'-----------------------------------------------------------
' QB Voxel Engine 2.2d w/ rotations
' by Toshihiro Horie 1998, DirectQB by Angelo Mottola.
' * observer's viewpoint was raised to a height of 80
' * If you get "Subprogram not defined", then you need to
'   load QB4.5 using QB /LC:\QB45\DQB\DQB given that DQB.LIB
'   is in C:\QB45\DQB. Also fix the path to DIRECTQB.BI.
' http://www.ocf.berkeley.edu/~horie/project.html
'-----------------------------------------------------------

DECLARE SUB Precalc ()
DECLARE SUB SetPal ()
'$INCLUDE: 'c:\qb45\dqb\DIRECTQB.BI'

CONST xmax = 320, ymax = 200: 'screen resolution
CONST xc = 160, yc = 50:     'center of rotation
CONST xh = 64, yh = 64:       'length and width of 2d heightfield
CONST xhc = 32, yhc = 32:     'half the width and height of heightfield
CONST dar = 43:               'dar controls the tallness of the voxels
CONST zmax = 83:              'clipping 'horizon'
CONST x0 = 40, x1 = 259:      'sprite dimension in x direction
CONST y0 = 88, y1 = 199:      'sprite dimension in y direction
CONST layer1 = 1:             'screen buffer handle for DQB.
mode = 2:       ' Mode 1: uses dots  -- fastest but not voxel graphics.
                ' Mode 2: uses dots  -- with height shadow
                ' Mode 3: uses boxes -- non-interpolated 'true' voxels.


DIM SHARED land(xh, yh)
DIM SHARED sine(64)
DIM SHARED cosine(64)
DIM SHARED c(128)
DIM SHARED s(128)
DIM SHARED xtable(10 TO zmax, -32 TO 31)
DIM SHARED ybottable(1 TO 180)
DIM SHARED ytoptable(1 TO zmax, -160 TO 20)


Precalc
asize = INT((4 + INT(((x1 - x0 + 1) * 8 + 7) / 8) * (y1 - y0 + 1)) / 2)
DIM Sprite(asize)

SCREEN 0: WIDTH 80: CLS
IF DQBinit(1, 0) THEN
   PRINT "DQBInit error: Need more EMS memory."
   PRINT "Add this to your C:\CONFIG.SYS file and reboot:"
   PRINT "   DEVICE=C:\WINDOWS\HIMEM.SYS"
   PRINT "   DEVICE=C:\WINDOWS\EMM386.EXE 2048 RAM"
   PRINT "Press any key to return to system."
   K$ = INPUT$(1)
   DQBclose
   END
END IF

SCREEN 13
GET (x0, y0)-(x1, y1), Sprite: 'safety check

DQBinitVGA
SetPal
DQBclearLayer layer1: 'CLS our buffer
DQBprint VIDEO, "QB VOXEL ENGINE 2.2 DQB", 106, 20, 123
DQBprint VIDEO, "by Toshihiro Horie", 124, 30, 123
DQBLine VIDEO, 97, 40, 250, 40, 123
DQBLine VIDEO, 250, 40, 250, 75, 123
DQBbox VIDEO, 203, 75, 271, 83, 123
'draw isometric 2-d color height map on right corner
FOR j = 0 TO xh - 1
FOR i = 0 TO xh - 1
   DQBLine VIDEO, i + 30, j + 22, i + 30 - land(i AND 63, j AND 63) \ 8, j + 22 - land(i AND 63, j AND 63) \ 10, land(i AND 63, j AND 63)
NEXT: NEXT

'-------------------------------------------
'Main loop switches between the 3 modes
'-------------------------------------------
FOR mode = 1 TO 4
   IF mode > 3 THEN mode = 3: 'infinite mode 3
   DQBprint VIDEO, "Mode " + STR$(mode - 1), 210, 76, 0
   DQBprint VIDEO, "Mode " + STR$(mode), 210, 76, 100
   FOR angle = 0 TO 127
      Ca = c(angle)
      Sa = s(angle)
      'clear window
      DQBboxf layer1, x0, y0, x1, y1, 0
      FOR z3d = (zmax - 10) TO 1 STEP -1
         z = z3d + 10
         yo = z3d - 31
         yrot2 = yo * Sa \ 512
         yrot1 = yo * Ca \ 512
         ybot = ybottable(z)
         FOR x3d = -32 TO 30
            'only needed if rotating mountains
            xr = x3d * Ca \ 512 + yrot2 + xhc
            yr = yrot1 + yhc - x3d * Sa \ 512
            'check if we are inside the boundaries of our heightmap
            IF xr < 0 OR xr > xh - 1 OR yr < 0 OR yr > yh - 1 THEN
               GOTO nodraw
            ELSE
               y3d = land(xr, yr)
            END IF
            colour = y3d
            ytop = ytoptable(z, y3d - 160)
            x = xtable(z, x3d)
            SELECT CASE mode
               CASE 1
                  DQBpset layer1, x, ytop, colour
               CASE 2
                  IF (xr = 0 OR xr = xh - 1) AND (yr = 0 OR yr = yh - 1) THEN
                     DQBLine layer1, x, ytop, x, ybot, 123
                  ELSE
                     DQBpset layer1, x, ybot, colour
                     DQBpset layer1, x, ytop, colour
                  END IF
               CASE 3
                  x2 = xtable(z, x3d + 1) - 1
                  IF ybot > 199 THEN ybot = 199
                  IF ytop > 199 OR x < 0 OR x2 < 0 OR x > 319 OR x2 > 319 THEN GOTO nodraw
                  DQBboxf layer1, x, ytop, x2, ybot, colour
            END SELECT
nodraw:
         NEXT x3d
         IF INKEY$ > "" THEN GOTO cleanup
      NEXT z3d
   DQBget layer1, x0, y0, x1, y1, VARSEG(Sprite(0)), VARPTR(Sprite(0))
   DQBsetSolidPut
   DQBfput VIDEO, x0, y0, VARSEG(Sprite(0)), VARPTR(Sprite(0))
   NEXT angle
NEXT mode




cleanup:
'go back to text mode
DQBfadeTo 0, 0, 0
DQBinitText
 
'DQB memory deallocation
DQBclose

REM $DYNAMIC
SUB Precalc
deg2rad# = 3.1415926535# / 32
FOR i = 0 TO 63
   sine(i) = INT(SIN(i * deg2rad#) * 40) + 40
   cosine(i) = INT(COS(i * deg2rad#) * 40) + 40
NEXT i

'set up sinus tables
pi# = 3.1415926535#
deg2bin# = 2 * pi# / 128
scale = 512
FOR theta = 0 TO 63
    s(theta) = CINT(SIN(theta * deg2bin#) * scale)
    c(theta) = CINT(COS(theta * deg2bin#) * scale)
NEXT
FOR theta = 64 TO 127
    s(theta) = -s(theta - 63)
    c(theta) = c(127 - theta)
NEXT

'precalc land
FOR j = 0 TO yh - 1
FOR i = 0 TO xh - 1
   land(i, j) = sine((i + 8) AND 63) + cosine(j AND 63)
NEXT: NEXT
'precalc
FOR z = 10 TO zmax
FOR x3d = -32 TO 31
   xtable(z, x3d) = xc * x3d / z + xc
NEXT: NEXT
'precalc ybot
FOR z = 1 TO 180
ybottable(z) = yc + 160 * dar / z
NEXT z
'precalc ytop
FOR z = 1 TO zmax
FOR y3d = -160 TO 20
      ytoptable(z, y3d) = yc - dar * y3d / z
NEXT: NEXT
END SUB

SUB SetPal
FOR i = 0 TO 63
OUT &H3C8, i + 0
OUT &H3C9, 0
OUT &H3C9, i
OUT &H3C9, 0
OUT &H3C8, i + 128
OUT &H3C9, i
OUT &H3C9, i \ 2
OUT &H3C9, 0
OUT &H3C8, i + 64
OUT &H3C9, i
OUT &H3C9, i
OUT &H3C9, i
OUT &H3C8, i + 176
OUT &H3C9, 63
OUT &H3C9, 63
OUT &H3C9, 63
NEXT i
FOR i = 0 TO 20
OUT &H3C8, i
OUT &H3C9, 0
OUT &H3C9, 0
OUT &H3C9, i + 10
NEXT i

OUT &H3C8, 0
OUT &H3C9, 0
OUT &H3C9, 0
OUT &H3C9, 0
END SUB

